import sys
import warnings
from getgauge.registry import registry, MessagesStore, ScreenshotsStore

if sys.version_info[0] is 3:
    from collections.abc import MutableMapping
else:
    from collections import MutableMapping


def step(step_text):
    def _step(func):
        f_code = sys._getframe().f_back.f_code
        span = {'start': f_code.co_firstlineno,
                'startChar': 0, 'end': 0, 'endChar': 0}
        registry.add_step(step_text, func, f_code.co_filename, span)
        return func

    return _step


def continue_on_failure(obj):
    return _define_wrapper(obj, registry.continue_on_failure)


def before_suite(obj=None):
    return _define_wrapper(obj, registry.add_before_suite)


def after_suite(obj=None):
    return _define_wrapper(obj, registry.add_after_suite)


def before_scenario(obj=None):
    return _define_wrapper(obj, registry.add_before_scenario)


def after_scenario(obj=None):
    return _define_wrapper(obj, registry.add_after_scenario)


def before_spec(obj=None):
    return _define_wrapper(obj, registry.add_before_spec)


def after_spec(obj=None):
    return _define_wrapper(obj, registry.add_after_spec)


def before_step(obj=None):
    return _define_wrapper(obj, registry.add_before_step)


def after_step(obj=None):
    return _define_wrapper(obj, registry.add_after_step)


def screenshot(func):
    _warn_screenshot_deprecation()
    registry.set_screenshot_provider(func)
    return func


def custom_screen_grabber(func):
    registry.set_screenshot_provider(func)
    return func


def _warn_screenshot_deprecation():
    warnings.warn(
        "'screenshot' is deprecated in favour of 'custom_screen_grabber'",
        DeprecationWarning, stacklevel=3)
    warnings.simplefilter('default', DeprecationWarning)


class Table:
    def __init__(self, proto_table):
        self.__headers = proto_table.headers.cells
        self.__rows = proto_table.rows

    @property
    def headers(self):
        return self.__headers

    @property
    def rows(self):
        return [row.cells for row in self.__rows]

    def get_row(self, index):
        return self.__rows[index - 1].cells

    def get_column_values_with_name(self, name):
        index = list(self.__headers).index(name)
        return self.get_column_values_with_index(index + 1)

    def get_column_values_with_index(self, index):
        return [row.cells[index - 1] for row in self.__rows]

    def __str__(self):
        table = [""] * (len(self.__rows) + 2)
        for header in self.__headers:
            values = [header, ""] + self.get_column_values_with_name(header)
            m_length = max(map(len, values))
            values[1] = "-" * m_length
            values = [value.ljust(m_length) for value in values]
            for i, value in enumerate(values):
                table[i] = table[i] or []
                table[i].append(value)
        return "\n".join(["|{}|".format("|".join(row)) for row in table])

    def __eq__(self, other):
        return self.__str__() == other.__str__()

    def __getitem__(self, i):
        return self.get_row(i + 1)

    def __iter__(self):
        return self.rows.__iter__()


class ExecutionContext:
    def __init__(self, specification, scenario, step):
        self.__step = step
        self.__specification = specification
        self.__scenario = scenario

    @property
    def specification(self):
        return self.__specification

    @property
    def scenario(self):
        return self.__scenario

    @property
    def step(self):
        return self.__step

    def __str__(self):
        return 'ExecutionInfo: {{ {}, {}, {} }}'.format(self.specification.__str__(), self.scenario.__str__(),
                                                        self.step.__str__())

    def __eq__(self, other):
        return self.__str__() == other.__str__()


class Specification:
    def __init__(self, name, file_name, is_failing, tags):
        self.__name = name
        self.__file_name = file_name
        self.__is_failing = is_failing
        self.__tags = tags

    @property
    def name(self):
        return self.__name

    @property
    def file_name(self):
        return self.__file_name

    @property
    def is_failing(self):
        return self.__is_failing

    @property
    def tags(self):
        return self.__tags

    def __str__(self):
        return "Specification: {{ name: {}, is_failing: {}, tags: {}, file_name: {} }}".format(self.name,
                                                                                               str(
                                                                                                   self.is_failing),
                                                                                               ", ".join(
                                                                                                   self.tags),
                                                                                               self.file_name)

    def __eq__(self, other):
        return self.__str__() == other.__str__()


class Scenario:
    def __init__(self, name, is_failing, tags):
        self.__name = name
        self.__is_failing = is_failing
        self.__tags = tags

    @property
    def name(self):
        return self.__name

    @property
    def is_failing(self):
        return self.__is_failing

    @property
    def tags(self):
        return self.__tags

    def __str__(self):
        return "Scenario: {{ name: {}, is_failing: {}, tags: {} }}".format(self.name, str(self.is_failing),
                                                                           ", ".join(self.tags))

    def __eq__(self, other):
        return self.__str__() == other.__str__()


class Step:
    def __init__(self, text, is_failing, message="", stacktrace=""):
        self.__stacktrace = stacktrace
        self.__error_message = message
        self.__text = text
        self.__is_failing = is_failing

    @property
    def text(self):
        return self.__text

    @property
    def is_failing(self):
        return self.__is_failing

    @property
    def error_message(self):
        return self.__error_message

    @property
    def stacktrace(self):
        return self.__stacktrace

    def __str__(self):
        s = "Step: {{ text: {}, is_failing: {}, error_message: {}, stacktrace: {} }}"
        return s.format(self.text, str(self.is_failing), str(self.error_message), str(self.stacktrace))

    def __eq__(self, other):
        return self.__str__() == other.__str__()


class Messages:
    @staticmethod
    def write_message(message):
        MessagesStore.write_message(message)


class DictObject(dict):
    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("'{0}' object has no attribute '{1}'".format(
                self.__class__.__name__, name))

    def __setattr__(self, name, value):
        self[name] = value

    def __delattr__(self, name):
        try:
            del self[name]
        except KeyError:
            raise AttributeError("'{0}' object has no attribute '{1}'".format(
                self.__class__.__name__, name))


class DataStoreContainer(object):
    def __init__(self):
        self.__scenario = DictObject()
        self.__spec = DictObject()
        self.__suite = DictObject()

    @property
    def scenario(self):
        return self.__scenario

    @property
    def spec(self):
        return self.__spec

    @property
    def suite(self):
        return self.__suite


data_store = DataStoreContainer()


class DataStore:
    def __init__(self, data_store=None):
        if data_store is None:
            data_store = {}
        self.__data_store = data_store

    def get(self, key):
        return self.__data_store[key]

    def put(self, key, value):
        self.__data_store[key] = value

    def is_present(self, key):
        return key in self.__data_store

    def clear(self):
        self.__data_store.clear()

    def __eq__(self, other):
        return self.__data_store == other.__data_store


def _warn_datastore_deprecation(store_type):
    warnings.warn(
        "'DataStoreFactory.{0}_data_store()' is deprecated in favour of 'data_store.{0}'".format(
            store_type),
        DeprecationWarning, stacklevel=3)
    warnings.simplefilter('default', DeprecationWarning)


class DataStoreFactory:
    __scenario_data_store = DataStore(data_store.scenario)
    __spec_data_store = DataStore(data_store.spec)
    __suite_data_store = DataStore(data_store.suite)

    @staticmethod
    def scenario_data_store():
        _warn_datastore_deprecation("scenario")
        return DataStoreFactory.__scenario_data_store

    @staticmethod
    def spec_data_store():
        _warn_datastore_deprecation("spec")
        return DataStoreFactory.__spec_data_store

    @staticmethod
    def suite_data_store():
        _warn_datastore_deprecation("suite")
        return DataStoreFactory.__suite_data_store


def create_execution_context_from(current_execution_info):
    return ExecutionContext(
        Specification(current_execution_info.currentSpec.name, current_execution_info.currentSpec.fileName,
                      current_execution_info.currentSpec.isFailed, current_execution_info.currentSpec.tags),
        Scenario(current_execution_info.currentScenario.name, current_execution_info.currentScenario.isFailed,
                 current_execution_info.currentScenario.tags),
        Step(current_execution_info.currentStep.step.actualStepText, current_execution_info.currentStep.isFailed,
             current_execution_info.currentStep.errorMessage, current_execution_info.currentStep.stackTrace)
    )


def _wrapper(*args, **kwargs):
    pass


def _define_wrapper(obj, callback):
    if hasattr(obj, '__call__'):
        callback(obj, None)
        return obj

    def func(function):
        callback(function, obj)
        return function

    return func


class Screenshots:
    @staticmethod
    def capture_screenshot():
        ScreenshotsStore.capture()



import inspect
import os
import sys
import time
import traceback

from getgauge.messages.messages_pb2 import Message
from getgauge.messages.spec_pb2 import ProtoExecutionResult
from getgauge.registry import registry, MessagesStore, ScreenshotsStore


def set_response_values(request, response):
    response.messageType = Message.ExecutionStatusResponse
    response.executionStatusResponse.executionResult.failed = False
    response.executionStatusResponse.executionResult.executionTime = 0


def run_hook(request, response, hooks, execution_info):
    set_response_values(request, response)
    for hook in hooks:
        execute_method([execution_info], hook, response)


def _false(func, exception): return False


def execute_method(params, step, response, is_continue_on_failure=_false):
    start = _current_time()
    try:
        params = _get_args(params, step)
        step.impl(*params)
    except Exception as e:
        _add_exception(e, response, is_continue_on_failure(step.impl, e))
    response.executionStatusResponse.executionResult.executionTime = _current_time() - start
    response.executionStatusResponse.executionResult.message.extend(MessagesStore.pending_messages())
    response.executionStatusResponse.executionResult.screenshots.extend(ScreenshotsStore.pending_screenshots())


def _current_time(): return int(round(time.time() * 1000))


def _get_args(params, hook_or_step):
    _params = [hook_or_step.instance] if hook_or_step.instance is not None else []
    args_length = 0
    if sys.version_info < (3, 3):
        args_length = len(inspect.getargspec(hook_or_step.impl).args)
    else:
        args_length = len(inspect.signature(hook_or_step.impl).parameters)
    if args_length < 1:
        return []
    return (_params + params) if args_length > len(params) else params

def _add_exception(e, response, continue_on_failure):
    if os.getenv('screenshot_on_failure') == 'true':
        screenshot = registry.screenshot_provider()()
        response.executionStatusResponse.executionResult.screenShot = screenshot
        response.executionStatusResponse.executionResult.failureScreenshot = screenshot
    response.executionStatusResponse.executionResult.failed = True
    message = e.__str__()
    if not message:
        message = "Exception occurred"
    response.executionStatusResponse.executionResult.errorMessage = message
    response.executionStatusResponse.executionResult.stackTrace = traceback.format_exc()
    response.executionStatusResponse.executionResult.errorType = ProtoExecutionResult.ASSERTION
    if continue_on_failure:
        response.executionStatusResponse.executionResult.recoverableError = True
    response.executionStatusResponse.executionResult.message.extend(MessagesStore.pending_messages())
    response.executionStatusResponse.executionResult.screenshots.extend(ScreenshotsStore.pending_screenshots())



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import getgauge.messages.spec_pb2 as spec__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='api.proto',
  package='gauge.messages',
  syntax='proto3',
  serialized_pb=_b('\n\tapi.proto\x12\x0egauge.messages\x1a\nspec.proto\"\x17\n\x15GetProjectRootRequest\"-\n\x16GetProjectRootResponse\x12\x13\n\x0bprojectRoot\x18\x01 \x01(\t\"\x1c\n\x1aGetInstallationRootRequest\"7\n\x1bGetInstallationRootResponse\x12\x18\n\x10installationRoot\x18\x01 \x01(\t\"\x14\n\x12GetAllStepsRequest\"G\n\x13GetAllStepsResponse\x12\x30\n\x08\x61llSteps\x18\x01 \x03(\x0b\x32\x1e.gauge.messages.ProtoStepValue\"\x1d\n\x0cSpecsRequest\x12\r\n\x05specs\x18\x01 \x03(\t\"\xad\x01\n\rSpecsResponse\x12\x39\n\x07\x64\x65tails\x18\x01 \x03(\x0b\x32(.gauge.messages.SpecsResponse.SpecDetail\x1a\x61\n\nSpecDetail\x12\'\n\x04spec\x18\x01 \x01(\x0b\x32\x19.gauge.messages.ProtoSpec\x12*\n\x0bparseErrors\x18\x02 \x03(\x0b\x32\x15.gauge.messages.Error\"\x17\n\x15GetAllConceptsRequest\"G\n\x16GetAllConceptsResponse\x12-\n\x08\x63oncepts\x18\x01 \x03(\x0b\x32\x1b.gauge.messages.ConceptInfo\"f\n\x0b\x43onceptInfo\x12\x31\n\tstepValue\x18\x01 \x01(\x0b\x32\x1e.gauge.messages.ProtoStepValue\x12\x10\n\x08\x66ilepath\x18\x02 \x01(\t\x12\x12\n\nlineNumber\x18\x03 \x01(\x05\"?\n\x13GetStepValueRequest\x12\x10\n\x08stepText\x18\x01 \x01(\t\x12\x16\n\x0ehasInlineTable\x18\x02 \x01(\x08\"I\n\x14GetStepValueResponse\x12\x31\n\tstepValue\x18\x01 \x01(\x0b\x32\x1e.gauge.messages.ProtoStepValue\"3\n\x1fGetLanguagePluginLibPathRequest\x12\x10\n\x08language\x18\x01 \x01(\t\"0\n GetLanguagePluginLibPathResponse\x12\x0c\n\x04path\x18\x01 \x01(\t\"\x1e\n\rErrorResponse\x12\r\n\x05\x65rror\x18\x01 \x01(\t\"=\n\x19PerformRefactoringRequest\x12\x0f\n\x07oldStep\x18\x01 \x01(\t\x12\x0f\n\x07newStep\x18\x02 \x01(\t\"S\n\x1aPerformRefactoringResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0e\n\x06\x65rrors\x18\x02 \x03(\t\x12\x14\n\x0c\x66ilesChanged\x18\x03 \x03(\t\"\xd1\x01\n\x15\x45xtractConceptRequest\x12)\n\x0b\x63onceptName\x18\x01 \x01(\x0b\x32\x14.gauge.messages.step\x12#\n\x05steps\x18\x02 \x03(\x0b\x32\x14.gauge.messages.step\x12\x1b\n\x13\x63hangeAcrossProject\x18\x03 \x01(\x08\x12\x17\n\x0f\x63onceptFileName\x18\x04 \x01(\t\x12\x32\n\x10selectedTextInfo\x18\x05 \x01(\x0b\x32\x18.gauge.messages.textInfo\"G\n\x08textInfo\x12\x10\n\x08\x66ileName\x18\x01 \x01(\t\x12\x16\n\x0estartingLineNo\x18\x02 \x01(\x05\x12\x11\n\tendLineNo\x18\x03 \x01(\x05\";\n\x04step\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05table\x18\x02 \x01(\t\x12\x16\n\x0eparamTableName\x18\x03 \x01(\t\"P\n\x16\x45xtractConceptResponse\x12\x11\n\tisSuccess\x18\x01 \x01(\x08\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x14\n\x0c\x66ilesChanged\x18\x03 \x03(\t\"#\n\x12\x46ormatSpecsRequest\x12\r\n\x05specs\x18\x01 \x03(\t\"7\n\x13\x46ormatSpecsResponse\x12\x0e\n\x06\x65rrors\x18\x01 \x03(\t\x12\x10\n\x08warnings\x18\x02 \x03(\t\"\x1f\n\x1dUnsupportedApiMessageResponse\"\xa9\x11\n\nAPIMessage\x12>\n\x0bmessageType\x18\x01 \x01(\x0e\x32).gauge.messages.APIMessage.APIMessageType\x12\x11\n\tmessageId\x18\x02 \x01(\x03\x12\x41\n\x12projectRootRequest\x18\x03 \x01(\x0b\x32%.gauge.messages.GetProjectRootRequest\x12\x43\n\x13projectRootResponse\x18\x04 \x01(\x0b\x32&.gauge.messages.GetProjectRootResponse\x12K\n\x17installationRootRequest\x18\x05 \x01(\x0b\x32*.gauge.messages.GetInstallationRootRequest\x12M\n\x18installationRootResponse\x18\x06 \x01(\x0b\x32+.gauge.messages.GetInstallationRootResponse\x12;\n\x0f\x61llStepsRequest\x18\x07 \x01(\x0b\x32\".gauge.messages.GetAllStepsRequest\x12=\n\x10\x61llStepsResponse\x18\x08 \x01(\x0b\x32#.gauge.messages.GetAllStepsResponse\x12\x32\n\x0cspecsRequest\x18\t \x01(\x0b\x32\x1c.gauge.messages.SpecsRequest\x12\x34\n\rspecsResponse\x18\n \x01(\x0b\x32\x1d.gauge.messages.SpecsResponse\x12=\n\x10stepValueRequest\x18\x0b \x01(\x0b\x32#.gauge.messages.GetStepValueRequest\x12?\n\x11stepValueResponse\x18\x0c \x01(\x0b\x32$.gauge.messages.GetStepValueResponse\x12G\n\x0elibPathRequest\x18\r \x01(\x0b\x32/.gauge.messages.GetLanguagePluginLibPathRequest\x12I\n\x0flibPathResponse\x18\x0e \x01(\x0b\x32\x30.gauge.messages.GetLanguagePluginLibPathResponse\x12,\n\x05\x65rror\x18\x0f \x01(\x0b\x32\x1d.gauge.messages.ErrorResponse\x12\x41\n\x12\x61llConceptsRequest\x18\x10 \x01(\x0b\x32%.gauge.messages.GetAllConceptsRequest\x12\x43\n\x13\x61llConceptsResponse\x18\x11 \x01(\x0b\x32&.gauge.messages.GetAllConceptsResponse\x12L\n\x19performRefactoringRequest\x18\x12 \x01(\x0b\x32).gauge.messages.PerformRefactoringRequest\x12N\n\x1aperformRefactoringResponse\x18\x13 \x01(\x0b\x32*.gauge.messages.PerformRefactoringResponse\x12\x44\n\x15\x65xtractConceptRequest\x18\x14 \x01(\x0b\x32%.gauge.messages.ExtractConceptRequest\x12\x46\n\x16\x65xtractConceptResponse\x18\x15 \x01(\x0b\x32&.gauge.messages.ExtractConceptResponse\x12>\n\x12\x66ormatSpecsRequest\x18\x16 \x01(\x0b\x32\".gauge.messages.FormatSpecsRequest\x12@\n\x13\x66ormatSpecsResponse\x18\x17 \x01(\x0b\x32#.gauge.messages.FormatSpecsResponse\x12T\n\x1dunsupportedApiMessageResponse\x18\x18 \x01(\x0b\x32-.gauge.messages.UnsupportedApiMessageResponse\"\xef\x04\n\x0e\x41PIMessageType\x12\x19\n\x15GetProjectRootRequest\x10\x00\x12\x1a\n\x16GetProjectRootResponse\x10\x01\x12\x1e\n\x1aGetInstallationRootRequest\x10\x02\x12\x1f\n\x1bGetInstallationRootResponse\x10\x03\x12\x16\n\x12GetAllStepsRequest\x10\x04\x12\x16\n\x12GetAllStepResponse\x10\x05\x12\x10\n\x0cSpecsRequest\x10\x06\x12\x11\n\rSpecsResponse\x10\x07\x12\x17\n\x13GetStepValueRequest\x10\x08\x12\x18\n\x14GetStepValueResponse\x10\t\x12#\n\x1fGetLanguagePluginLibPathRequest\x10\n\x12$\n GetLanguagePluginLibPathResponse\x10\x0b\x12\x11\n\rErrorResponse\x10\x0c\x12\x19\n\x15GetAllConceptsRequest\x10\r\x12\x1a\n\x16GetAllConceptsResponse\x10\x0e\x12\x1d\n\x19PerformRefactoringRequest\x10\x0f\x12\x1e\n\x1aPerformRefactoringResponse\x10\x10\x12\x19\n\x15\x45xtractConceptRequest\x10\x11\x12\x1a\n\x16\x45xtractConceptResponse\x10\x12\x12\x16\n\x12\x46ormatSpecsRequest\x10\x13\x12\x17\n\x13\x46ormatSpecsResponse\x10\x14\x12!\n\x1dUnsupportedApiMessageResponse\x10\x15\x42)\n\x16\x63om.thoughtworks.gauge\xaa\x02\x0eGauge.Messagesb\x06proto3')
  ,
  dependencies=[spec__pb2.DESCRIPTOR,])



_APIMESSAGE_APIMESSAGETYPE = _descriptor.EnumDescriptor(
  name='APIMessageType',
  full_name='gauge.messages.APIMessage.APIMessageType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='GetProjectRootRequest', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetProjectRootResponse', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetInstallationRootRequest', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetInstallationRootResponse', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetAllStepsRequest', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetAllStepResponse', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SpecsRequest', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SpecsResponse', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetStepValueRequest', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetStepValueResponse', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetLanguagePluginLibPathRequest', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetLanguagePluginLibPathResponse', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ErrorResponse', index=12, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetAllConceptsRequest', index=13, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GetAllConceptsResponse', index=14, number=14,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PerformRefactoringRequest', index=15, number=15,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PerformRefactoringResponse', index=16, number=16,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ExtractConceptRequest', index=17, number=17,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ExtractConceptResponse', index=18, number=18,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FormatSpecsRequest', index=19, number=19,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FormatSpecsResponse', index=20, number=20,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UnsupportedApiMessageResponse', index=21, number=21,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3277,
  serialized_end=3900,
)
_sym_db.RegisterEnumDescriptor(_APIMESSAGE_APIMESSAGETYPE)


_GETPROJECTROOTREQUEST = _descriptor.Descriptor(
  name='GetProjectRootRequest',
  full_name='gauge.messages.GetProjectRootRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=64,
)


_GETPROJECTROOTRESPONSE = _descriptor.Descriptor(
  name='GetProjectRootResponse',
  full_name='gauge.messages.GetProjectRootResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projectRoot', full_name='gauge.messages.GetProjectRootResponse.projectRoot', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=111,
)


_GETINSTALLATIONROOTREQUEST = _descriptor.Descriptor(
  name='GetInstallationRootRequest',
  full_name='gauge.messages.GetInstallationRootRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=113,
  serialized_end=141,
)


_GETINSTALLATIONROOTRESPONSE = _descriptor.Descriptor(
  name='GetInstallationRootResponse',
  full_name='gauge.messages.GetInstallationRootResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='installationRoot', full_name='gauge.messages.GetInstallationRootResponse.installationRoot', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=143,
  serialized_end=198,
)


_GETALLSTEPSREQUEST = _descriptor.Descriptor(
  name='GetAllStepsRequest',
  full_name='gauge.messages.GetAllStepsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=200,
  serialized_end=220,
)


_GETALLSTEPSRESPONSE = _descriptor.Descriptor(
  name='GetAllStepsResponse',
  full_name='gauge.messages.GetAllStepsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='allSteps', full_name='gauge.messages.GetAllStepsResponse.allSteps', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=222,
  serialized_end=293,
)


_SPECSREQUEST = _descriptor.Descriptor(
  name='SpecsRequest',
  full_name='gauge.messages.SpecsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='specs', full_name='gauge.messages.SpecsRequest.specs', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=295,
  serialized_end=324,
)


_SPECSRESPONSE_SPECDETAIL = _descriptor.Descriptor(
  name='SpecDetail',
  full_name='gauge.messages.SpecsResponse.SpecDetail',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='spec', full_name='gauge.messages.SpecsResponse.SpecDetail.spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parseErrors', full_name='gauge.messages.SpecsResponse.SpecDetail.parseErrors', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=403,
  serialized_end=500,
)

_SPECSRESPONSE = _descriptor.Descriptor(
  name='SpecsResponse',
  full_name='gauge.messages.SpecsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='details', full_name='gauge.messages.SpecsResponse.details', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SPECSRESPONSE_SPECDETAIL, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=327,
  serialized_end=500,
)


_GETALLCONCEPTSREQUEST = _descriptor.Descriptor(
  name='GetAllConceptsRequest',
  full_name='gauge.messages.GetAllConceptsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=502,
  serialized_end=525,
)


_GETALLCONCEPTSRESPONSE = _descriptor.Descriptor(
  name='GetAllConceptsResponse',
  full_name='gauge.messages.GetAllConceptsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='concepts', full_name='gauge.messages.GetAllConceptsResponse.concepts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=527,
  serialized_end=598,
)


_CONCEPTINFO = _descriptor.Descriptor(
  name='ConceptInfo',
  full_name='gauge.messages.ConceptInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stepValue', full_name='gauge.messages.ConceptInfo.stepValue', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filepath', full_name='gauge.messages.ConceptInfo.filepath', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lineNumber', full_name='gauge.messages.ConceptInfo.lineNumber', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=600,
  serialized_end=702,
)


_GETSTEPVALUEREQUEST = _descriptor.Descriptor(
  name='GetStepValueRequest',
  full_name='gauge.messages.GetStepValueRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stepText', full_name='gauge.messages.GetStepValueRequest.stepText', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasInlineTable', full_name='gauge.messages.GetStepValueRequest.hasInlineTable', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=704,
  serialized_end=767,
)


_GETSTEPVALUERESPONSE = _descriptor.Descriptor(
  name='GetStepValueResponse',
  full_name='gauge.messages.GetStepValueResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stepValue', full_name='gauge.messages.GetStepValueResponse.stepValue', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=769,
  serialized_end=842,
)


_GETLANGUAGEPLUGINLIBPATHREQUEST = _descriptor.Descriptor(
  name='GetLanguagePluginLibPathRequest',
  full_name='gauge.messages.GetLanguagePluginLibPathRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='language', full_name='gauge.messages.GetLanguagePluginLibPathRequest.language', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=844,
  serialized_end=895,
)


_GETLANGUAGEPLUGINLIBPATHRESPONSE = _descriptor.Descriptor(
  name='GetLanguagePluginLibPathResponse',
  full_name='gauge.messages.GetLanguagePluginLibPathResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='gauge.messages.GetLanguagePluginLibPathResponse.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=897,
  serialized_end=945,
)


_ERRORRESPONSE = _descriptor.Descriptor(
  name='ErrorResponse',
  full_name='gauge.messages.ErrorResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error', full_name='gauge.messages.ErrorResponse.error', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=947,
  serialized_end=977,
)


_PERFORMREFACTORINGREQUEST = _descriptor.Descriptor(
  name='PerformRefactoringRequest',
  full_name='gauge.messages.PerformRefactoringRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='oldStep', full_name='gauge.messages.PerformRefactoringRequest.oldStep', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='newStep', full_name='gauge.messages.PerformRefactoringRequest.newStep', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=979,
  serialized_end=1040,
)


_PERFORMREFACTORINGRESPONSE = _descriptor.Descriptor(
  name='PerformRefactoringResponse',
  full_name='gauge.messages.PerformRefactoringResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='gauge.messages.PerformRefactoringResponse.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errors', full_name='gauge.messages.PerformRefactoringResponse.errors', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filesChanged', full_name='gauge.messages.PerformRefactoringResponse.filesChanged', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1042,
  serialized_end=1125,
)


_EXTRACTCONCEPTREQUEST = _descriptor.Descriptor(
  name='ExtractConceptRequest',
  full_name='gauge.messages.ExtractConceptRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='conceptName', full_name='gauge.messages.ExtractConceptRequest.conceptName', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='steps', full_name='gauge.messages.ExtractConceptRequest.steps', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='changeAcrossProject', full_name='gauge.messages.ExtractConceptRequest.changeAcrossProject', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='conceptFileName', full_name='gauge.messages.ExtractConceptRequest.conceptFileName', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selectedTextInfo', full_name='gauge.messages.ExtractConceptRequest.selectedTextInfo', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1128,
  serialized_end=1337,
)


_TEXTINFO = _descriptor.Descriptor(
  name='textInfo',
  full_name='gauge.messages.textInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fileName', full_name='gauge.messages.textInfo.fileName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='startingLineNo', full_name='gauge.messages.textInfo.startingLineNo', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endLineNo', full_name='gauge.messages.textInfo.endLineNo', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1339,
  serialized_end=1410,
)


_STEP = _descriptor.Descriptor(
  name='step',
  full_name='gauge.messages.step',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='gauge.messages.step.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='table', full_name='gauge.messages.step.table', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='paramTableName', full_name='gauge.messages.step.paramTableName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1412,
  serialized_end=1471,
)


_EXTRACTCONCEPTRESPONSE = _descriptor.Descriptor(
  name='ExtractConceptResponse',
  full_name='gauge.messages.ExtractConceptResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='isSuccess', full_name='gauge.messages.ExtractConceptResponse.isSuccess', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='gauge.messages.ExtractConceptResponse.error', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filesChanged', full_name='gauge.messages.ExtractConceptResponse.filesChanged', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1473,
  serialized_end=1553,
)


_FORMATSPECSREQUEST = _descriptor.Descriptor(
  name='FormatSpecsRequest',
  full_name='gauge.messages.FormatSpecsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='specs', full_name='gauge.messages.FormatSpecsRequest.specs', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1555,
  serialized_end=1590,
)


_FORMATSPECSRESPONSE = _descriptor.Descriptor(
  name='FormatSpecsResponse',
  full_name='gauge.messages.FormatSpecsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='errors', full_name='gauge.messages.FormatSpecsResponse.errors', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='warnings', full_name='gauge.messages.FormatSpecsResponse.warnings', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1592,
  serialized_end=1647,
)


_UNSUPPORTEDAPIMESSAGERESPONSE = _descriptor.Descriptor(
  name='UnsupportedApiMessageResponse',
  full_name='gauge.messages.UnsupportedApiMessageResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1649,
  serialized_end=1680,
)


_APIMESSAGE = _descriptor.Descriptor(
  name='APIMessage',
  full_name='gauge.messages.APIMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='messageType', full_name='gauge.messages.APIMessage.messageType', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='messageId', full_name='gauge.messages.APIMessage.messageId', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='projectRootRequest', full_name='gauge.messages.APIMessage.projectRootRequest', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='projectRootResponse', full_name='gauge.messages.APIMessage.projectRootResponse', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='installationRootRequest', full_name='gauge.messages.APIMessage.installationRootRequest', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='installationRootResponse', full_name='gauge.messages.APIMessage.installationRootResponse', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='allStepsRequest', full_name='gauge.messages.APIMessage.allStepsRequest', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='allStepsResponse', full_name='gauge.messages.APIMessage.allStepsResponse', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='specsRequest', full_name='gauge.messages.APIMessage.specsRequest', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='specsResponse', full_name='gauge.messages.APIMessage.specsResponse', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stepValueRequest', full_name='gauge.messages.APIMessage.stepValueRequest', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stepValueResponse', full_name='gauge.messages.APIMessage.stepValueResponse', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='libPathRequest', full_name='gauge.messages.APIMessage.libPathRequest', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='libPathResponse', full_name='gauge.messages.APIMessage.libPathResponse', index=13,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='gauge.messages.APIMessage.error', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='allConceptsRequest', full_name='gauge.messages.APIMessage.allConceptsRequest', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='allConceptsResponse', full_name='gauge.messages.APIMessage.allConceptsResponse', index=16,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='performRefactoringRequest', full_name='gauge.messages.APIMessage.performRefactoringRequest', index=17,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='performRefactoringResponse', full_name='gauge.messages.APIMessage.performRefactoringResponse', index=18,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extractConceptRequest', full_name='gauge.messages.APIMessage.extractConceptRequest', index=19,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extractConceptResponse', full_name='gauge.messages.APIMessage.extractConceptResponse', index=20,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='formatSpecsRequest', full_name='gauge.messages.APIMessage.formatSpecsRequest', index=21,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='formatSpecsResponse', full_name='gauge.messages.APIMessage.formatSpecsResponse', index=22,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unsupportedApiMessageResponse', full_name='gauge.messages.APIMessage.unsupportedApiMessageResponse', index=23,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _APIMESSAGE_APIMESSAGETYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1683,
  serialized_end=3900,
)

_GETALLSTEPSRESPONSE.fields_by_name['allSteps'].message_type = spec__pb2._PROTOSTEPVALUE
_SPECSRESPONSE_SPECDETAIL.fields_by_name['spec'].message_type = spec__pb2._PROTOSPEC
_SPECSRESPONSE_SPECDETAIL.fields_by_name['parseErrors'].message_type = spec__pb2._ERROR
_SPECSRESPONSE_SPECDETAIL.containing_type = _SPECSRESPONSE
_SPECSRESPONSE.fields_by_name['details'].message_type = _SPECSRESPONSE_SPECDETAIL
_GETALLCONCEPTSRESPONSE.fields_by_name['concepts'].message_type = _CONCEPTINFO
_CONCEPTINFO.fields_by_name['stepValue'].message_type = spec__pb2._PROTOSTEPVALUE
_GETSTEPVALUERESPONSE.fields_by_name['stepValue'].message_type = spec__pb2._PROTOSTEPVALUE
_EXTRACTCONCEPTREQUEST.fields_by_name['conceptName'].message_type = _STEP
_EXTRACTCONCEPTREQUEST.fields_by_name['steps'].message_type = _STEP
_EXTRACTCONCEPTREQUEST.fields_by_name['selectedTextInfo'].message_type = _TEXTINFO
_APIMESSAGE.fields_by_name['messageType'].enum_type = _APIMESSAGE_APIMESSAGETYPE
_APIMESSAGE.fields_by_name['projectRootRequest'].message_type = _GETPROJECTROOTREQUEST
_APIMESSAGE.fields_by_name['projectRootResponse'].message_type = _GETPROJECTROOTRESPONSE
_APIMESSAGE.fields_by_name['installationRootRequest'].message_type = _GETINSTALLATIONROOTREQUEST
_APIMESSAGE.fields_by_name['installationRootResponse'].message_type = _GETINSTALLATIONROOTRESPONSE
_APIMESSAGE.fields_by_name['allStepsRequest'].message_type = _GETALLSTEPSREQUEST
_APIMESSAGE.fields_by_name['allStepsResponse'].message_type = _GETALLSTEPSRESPONSE
_APIMESSAGE.fields_by_name['specsRequest'].message_type = _SPECSREQUEST
_APIMESSAGE.fields_by_name['specsResponse'].message_type = _SPECSRESPONSE
_APIMESSAGE.fields_by_name['stepValueRequest'].message_type = _GETSTEPVALUEREQUEST
_APIMESSAGE.fields_by_name['stepValueResponse'].message_type = _GETSTEPVALUERESPONSE
_APIMESSAGE.fields_by_name['libPathRequest'].message_type = _GETLANGUAGEPLUGINLIBPATHREQUEST
_APIMESSAGE.fields_by_name['libPathResponse'].message_type = _GETLANGUAGEPLUGINLIBPATHRESPONSE
_APIMESSAGE.fields_by_name['error'].message_type = _ERRORRESPONSE
_APIMESSAGE.fields_by_name['allConceptsRequest'].message_type = _GETALLCONCEPTSREQUEST
_APIMESSAGE.fields_by_name['allConceptsResponse'].message_type = _GETALLCONCEPTSRESPONSE
_APIMESSAGE.fields_by_name['performRefactoringRequest'].message_type = _PERFORMREFACTORINGREQUEST
_APIMESSAGE.fields_by_name['performRefactoringResponse'].message_type = _PERFORMREFACTORINGRESPONSE
_APIMESSAGE.fields_by_name['extractConceptRequest'].message_type = _EXTRACTCONCEPTREQUEST
_APIMESSAGE.fields_by_name['extractConceptResponse'].message_type = _EXTRACTCONCEPTRESPONSE
_APIMESSAGE.fields_by_name['formatSpecsRequest'].message_type = _FORMATSPECSREQUEST
_APIMESSAGE.fields_by_name['formatSpecsResponse'].message_type = _FORMATSPECSRESPONSE
_APIMESSAGE.fields_by_name['unsupportedApiMessageResponse'].message_type = _UNSUPPORTEDAPIMESSAGERESPONSE
_APIMESSAGE_APIMESSAGETYPE.containing_type = _APIMESSAGE
DESCRIPTOR.message_types_by_name['GetProjectRootRequest'] = _GETPROJECTROOTREQUEST
DESCRIPTOR.message_types_by_name['GetProjectRootResponse'] = _GETPROJECTROOTRESPONSE
DESCRIPTOR.message_types_by_name['GetInstallationRootRequest'] = _GETINSTALLATIONROOTREQUEST
DESCRIPTOR.message_types_by_name['GetInstallationRootResponse'] = _GETINSTALLATIONROOTRESPONSE
DESCRIPTOR.message_types_by_name['GetAllStepsRequest'] = _GETALLSTEPSREQUEST
DESCRIPTOR.message_types_by_name['GetAllStepsResponse'] = _GETALLSTEPSRESPONSE
DESCRIPTOR.message_types_by_name['SpecsRequest'] = _SPECSREQUEST
DESCRIPTOR.message_types_by_name['SpecsResponse'] = _SPECSRESPONSE
DESCRIPTOR.message_types_by_name['GetAllConceptsRequest'] = _GETALLCONCEPTSREQUEST
DESCRIPTOR.message_types_by_name['GetAllConceptsResponse'] = _GETALLCONCEPTSRESPONSE
DESCRIPTOR.message_types_by_name['ConceptInfo'] = _CONCEPTINFO
DESCRIPTOR.message_types_by_name['GetStepValueRequest'] = _GETSTEPVALUEREQUEST
DESCRIPTOR.message_types_by_name['GetStepValueResponse'] = _GETSTEPVALUERESPONSE
DESCRIPTOR.message_types_by_name['GetLanguagePluginLibPathRequest'] = _GETLANGUAGEPLUGINLIBPATHREQUEST
DESCRIPTOR.message_types_by_name['GetLanguagePluginLibPathResponse'] = _GETLANGUAGEPLUGINLIBPATHRESPONSE
DESCRIPTOR.message_types_by_name['ErrorResponse'] = _ERRORRESPONSE
DESCRIPTOR.message_types_by_name['PerformRefactoringRequest'] = _PERFORMREFACTORINGREQUEST
DESCRIPTOR.message_types_by_name['PerformRefactoringResponse'] = _PERFORMREFACTORINGRESPONSE
DESCRIPTOR.message_types_by_name['ExtractConceptRequest'] = _EXTRACTCONCEPTREQUEST
DESCRIPTOR.message_types_by_name['textInfo'] = _TEXTINFO
DESCRIPTOR.message_types_by_name['step'] = _STEP
DESCRIPTOR.message_types_by_name['ExtractConceptResponse'] = _EXTRACTCONCEPTRESPONSE
DESCRIPTOR.message_types_by_name['FormatSpecsRequest'] = _FORMATSPECSREQUEST
DESCRIPTOR.message_types_by_name['FormatSpecsResponse'] = _FORMATSPECSRESPONSE
DESCRIPTOR.message_types_by_name['UnsupportedApiMessageResponse'] = _UNSUPPORTEDAPIMESSAGERESPONSE
DESCRIPTOR.message_types_by_name['APIMessage'] = _APIMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetProjectRootRequest = _reflection.GeneratedProtocolMessageType('GetProjectRootRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETPROJECTROOTREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetProjectRootRequest)
  ))
_sym_db.RegisterMessage(GetProjectRootRequest)

GetProjectRootResponse = _reflection.GeneratedProtocolMessageType('GetProjectRootResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETPROJECTROOTRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetProjectRootResponse)
  ))
_sym_db.RegisterMessage(GetProjectRootResponse)

GetInstallationRootRequest = _reflection.GeneratedProtocolMessageType('GetInstallationRootRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETINSTALLATIONROOTREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetInstallationRootRequest)
  ))
_sym_db.RegisterMessage(GetInstallationRootRequest)

GetInstallationRootResponse = _reflection.GeneratedProtocolMessageType('GetInstallationRootResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETINSTALLATIONROOTRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetInstallationRootResponse)
  ))
_sym_db.RegisterMessage(GetInstallationRootResponse)

GetAllStepsRequest = _reflection.GeneratedProtocolMessageType('GetAllStepsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETALLSTEPSREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetAllStepsRequest)
  ))
_sym_db.RegisterMessage(GetAllStepsRequest)

GetAllStepsResponse = _reflection.GeneratedProtocolMessageType('GetAllStepsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETALLSTEPSRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetAllStepsResponse)
  ))
_sym_db.RegisterMessage(GetAllStepsResponse)

SpecsRequest = _reflection.GeneratedProtocolMessageType('SpecsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SPECSREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.SpecsRequest)
  ))
_sym_db.RegisterMessage(SpecsRequest)

SpecsResponse = _reflection.GeneratedProtocolMessageType('SpecsResponse', (_message.Message,), dict(

  SpecDetail = _reflection.GeneratedProtocolMessageType('SpecDetail', (_message.Message,), dict(
    DESCRIPTOR = _SPECSRESPONSE_SPECDETAIL,
    __module__ = 'api_pb2'
    # @@protoc_insertion_point(class_scope:gauge.messages.SpecsResponse.SpecDetail)
    ))
  ,
  DESCRIPTOR = _SPECSRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.SpecsResponse)
  ))
_sym_db.RegisterMessage(SpecsResponse)
_sym_db.RegisterMessage(SpecsResponse.SpecDetail)

GetAllConceptsRequest = _reflection.GeneratedProtocolMessageType('GetAllConceptsRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETALLCONCEPTSREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetAllConceptsRequest)
  ))
_sym_db.RegisterMessage(GetAllConceptsRequest)

GetAllConceptsResponse = _reflection.GeneratedProtocolMessageType('GetAllConceptsResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETALLCONCEPTSRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetAllConceptsResponse)
  ))
_sym_db.RegisterMessage(GetAllConceptsResponse)

ConceptInfo = _reflection.GeneratedProtocolMessageType('ConceptInfo', (_message.Message,), dict(
  DESCRIPTOR = _CONCEPTINFO,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.ConceptInfo)
  ))
_sym_db.RegisterMessage(ConceptInfo)

GetStepValueRequest = _reflection.GeneratedProtocolMessageType('GetStepValueRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETSTEPVALUEREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetStepValueRequest)
  ))
_sym_db.RegisterMessage(GetStepValueRequest)

GetStepValueResponse = _reflection.GeneratedProtocolMessageType('GetStepValueResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETSTEPVALUERESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetStepValueResponse)
  ))
_sym_db.RegisterMessage(GetStepValueResponse)

GetLanguagePluginLibPathRequest = _reflection.GeneratedProtocolMessageType('GetLanguagePluginLibPathRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETLANGUAGEPLUGINLIBPATHREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetLanguagePluginLibPathRequest)
  ))
_sym_db.RegisterMessage(GetLanguagePluginLibPathRequest)

GetLanguagePluginLibPathResponse = _reflection.GeneratedProtocolMessageType('GetLanguagePluginLibPathResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETLANGUAGEPLUGINLIBPATHRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.GetLanguagePluginLibPathResponse)
  ))
_sym_db.RegisterMessage(GetLanguagePluginLibPathResponse)

ErrorResponse = _reflection.GeneratedProtocolMessageType('ErrorResponse', (_message.Message,), dict(
  DESCRIPTOR = _ERRORRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.ErrorResponse)
  ))
_sym_db.RegisterMessage(ErrorResponse)

PerformRefactoringRequest = _reflection.GeneratedProtocolMessageType('PerformRefactoringRequest', (_message.Message,), dict(
  DESCRIPTOR = _PERFORMREFACTORINGREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.PerformRefactoringRequest)
  ))
_sym_db.RegisterMessage(PerformRefactoringRequest)

PerformRefactoringResponse = _reflection.GeneratedProtocolMessageType('PerformRefactoringResponse', (_message.Message,), dict(
  DESCRIPTOR = _PERFORMREFACTORINGRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.PerformRefactoringResponse)
  ))
_sym_db.RegisterMessage(PerformRefactoringResponse)

ExtractConceptRequest = _reflection.GeneratedProtocolMessageType('ExtractConceptRequest', (_message.Message,), dict(
  DESCRIPTOR = _EXTRACTCONCEPTREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.ExtractConceptRequest)
  ))
_sym_db.RegisterMessage(ExtractConceptRequest)

textInfo = _reflection.GeneratedProtocolMessageType('textInfo', (_message.Message,), dict(
  DESCRIPTOR = _TEXTINFO,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.textInfo)
  ))
_sym_db.RegisterMessage(textInfo)

step = _reflection.GeneratedProtocolMessageType('step', (_message.Message,), dict(
  DESCRIPTOR = _STEP,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.step)
  ))
_sym_db.RegisterMessage(step)

ExtractConceptResponse = _reflection.GeneratedProtocolMessageType('ExtractConceptResponse', (_message.Message,), dict(
  DESCRIPTOR = _EXTRACTCONCEPTRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.ExtractConceptResponse)
  ))
_sym_db.RegisterMessage(ExtractConceptResponse)

FormatSpecsRequest = _reflection.GeneratedProtocolMessageType('FormatSpecsRequest', (_message.Message,), dict(
  DESCRIPTOR = _FORMATSPECSREQUEST,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.FormatSpecsRequest)
  ))
_sym_db.RegisterMessage(FormatSpecsRequest)

FormatSpecsResponse = _reflection.GeneratedProtocolMessageType('FormatSpecsResponse', (_message.Message,), dict(
  DESCRIPTOR = _FORMATSPECSRESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.FormatSpecsResponse)
  ))
_sym_db.RegisterMessage(FormatSpecsResponse)

UnsupportedApiMessageResponse = _reflection.GeneratedProtocolMessageType('UnsupportedApiMessageResponse', (_message.Message,), dict(
  DESCRIPTOR = _UNSUPPORTEDAPIMESSAGERESPONSE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.UnsupportedApiMessageResponse)
  ))
_sym_db.RegisterMessage(UnsupportedApiMessageResponse)

APIMessage = _reflection.GeneratedProtocolMessageType('APIMessage', (_message.Message,), dict(
  DESCRIPTOR = _APIMESSAGE,
  __module__ = 'api_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.APIMessage)
  ))
_sym_db.RegisterMessage(APIMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\026com.thoughtworks.gauge\252\002\016Gauge.Messages'))
# @@protoc_insertion_point(module_scope)



import os

PROJECT_ROOT_ENV = 'GAUGE_PROJECT_ROOT'
STEP_IMPL_DIR_ENV = 'STEP_IMPL_DIR'


def get_project_root():
    try:
        return os.path.abspath(os.environ[PROJECT_ROOT_ENV])
    except KeyError:
        return ""


def get_step_impl_dirs():
    step_impl_dir_names = map(str.strip, os.getenv(STEP_IMPL_DIR_ENV).split(',')) if os.getenv(STEP_IMPL_DIR_ENV) else ['step_impl']
    full_path_dir_names = []
    for name in step_impl_dir_names:
        name = name.replace("/", os.path.sep).replace("\\", os.path.sep)
        imple_dir = name if os.path.isabs(name) else os.path.join(get_project_root(), name)
        full_path_dir_names.append(imple_dir)
    return full_path_dir_names


def get_impl_files():
    step_impl_dirs = get_step_impl_dirs()
    file_list = []
    for step_impl_dir in step_impl_dirs:
        for root, _, files in os.walk(step_impl_dir):
            for file in files:
                if file.endswith('.py') and '__init__.py' != os.path.basename(file):
                    file_list.append(os.path.join(root, file))
    return file_list


def read_file_contents(file_name):
    if os.path.isfile(file_name):
        f = open(file_name)
        content = f.read().replace('\r\n', '\n')
        f.close()
        return content
    return None


def get_file_name(prefix='', counter=0):
    name = 'step_implementation{}.py'.format(prefix)
    file_name = os.path.join(get_step_impl_dirs()[0], name)
    if not os.path.exists(file_name):
        return file_name
    else:
        counter = counter + 1
        return get_file_name('_{}'.format(counter), counter)



import inspect
import os
import re
import sys
import tempfile
from subprocess import call

from getgauge import logger


class StepInfo(object):
    def __init__(self, step_text, parsed_step_text, impl, file_name, span, has_alias=False, aliases=None):
        if aliases is None:
            aliases = []
        self.__step_text, self.__parsed_step_text, self.__impl = step_text, parsed_step_text, impl
        self.__file_name, self.__span, self.__has_alias = file_name, span, has_alias
        self.__instance = None
        self.__aliases = aliases

    @property
    def step_text(self):
        return self.__step_text

    @property
    def parsed_step_text(self):
        return self.__parsed_step_text

    @property
    def impl(self):
        return self.__impl

    @property
    def instance(self):
        return self.__instance

    @instance.setter
    def instance(self, value):
        self.__instance = value

    @property
    def has_alias(self):
        return self.__has_alias

    @property
    def aliases(self):
        return self.__aliases

    @property
    def file_name(self):
        return self.__file_name

    @property
    def span(self):
        # If span is callable, lazy load span on access
        if callable(self.__span):
            self.__span = self.__span()
        return self.__span


class HookInfo(object):
    def __init__(self, tags, impl, file_name):
        self.__tags, self.__impl, self.__file_name = tags, impl, file_name
        self.__instance = None

    @property
    def tags(self):
        return self.__tags

    @property
    def impl(self):
        return self.__impl

    @property
    def file_name(self):
        return self.__file_name

    @property
    def instance(self):
        return self.__instance

    @instance.setter
    def instance(self, value):
        self.__instance = value


class MessagesStore:
    __messages = []

    @staticmethod
    def pending_messages():
        messages = MessagesStore.__messages
        MessagesStore.__messages = []
        return messages

    @staticmethod
    def write_message(message):
        MessagesStore.__messages.append(str(message))

    @staticmethod
    def clear():
        MessagesStore.__messages = []


class Registry(object):
    hooks = ['before_step', 'after_step', 'before_scenario', 'after_scenario', 'before_spec', 'after_spec',
             'before_suite', 'after_suite']

    def __init__(self):
        self.__screenshot_provider, self.__steps_map, self.__continue_on_failures = _take_screenshot, {}, {}
        for hook in Registry.hooks:
            self.__def_hook(hook)

    def __def_hook(self, hook):
        def get(self, tags=None):
            return _filter_hooks(tags, getattr(self, '__{}'.format(hook)))

        def add(self, func=None, tags=None, file_name=""):
            if not isinstance(func, str):
                file_name = inspect.getsourcefile(func)
            getattr(self, '__{}'.format(hook)).append(
                HookInfo(tags, func, file_name))

        setattr(self.__class__, hook, get)
        setattr(self.__class__, 'add_{}'.format(hook), add)
        setattr(self, '__{}'.format(hook), [])

    def add_step(self, step_text, func, file_name, span=None, has_alias=False, aliases=None):
        if not isinstance(step_text, list):
            parsed_step_text = _get_step_value(step_text)
            info = StepInfo(step_text, parsed_step_text, func,
                            file_name, span, has_alias, aliases)
            self.__steps_map.setdefault(parsed_step_text, []).append(info)
            return
        for text in step_text:
            self.add_step(text, func, file_name, span, True, step_text)

    def steps(self):
        return [value[0].step_text for value in self.__steps_map.values()]

    def is_implemented(self, step_text):
        return self.__steps_map.get(step_text) is not None

    def has_multiple_impls(self, step_text):
        return len(self.__steps_map.get(step_text)) > 1

    def get_info_for(self, step_text):
        info = self.__steps_map.get(step_text)
        return info[0] if info is not None else StepInfo(None, None, None, None, None)

    def get_infos_for(self, step_text):
        return self.__steps_map.get(step_text)

    def set_screenshot_provider(self, func):
        self.__screenshot_provider = func

    def screenshot_provider(self):
        return self.__screenshot_provider

    def continue_on_failure(self, func, exceptions=None):
        self.__continue_on_failures[func] = exceptions or [AssertionError]

    def is_continue_on_failure(self, func, exception):
        if func in self.__continue_on_failures:
            for e in self.__continue_on_failures[func]:
                if issubclass(type(exception), e):
                    return True
        return False

    def get_step_positions(self, file_name):
        positions = []
        for step, infos in self.__steps_map.items():
            positions = positions + [{'stepValue': step, 'span': i.span}
                                     for i in infos if i.file_name == file_name]
        return positions

    def _get_all_hooks(self, file_name):
        all_hooks = []
        for hook in self.hooks:
            all_hooks = all_hooks + \
                        [h for h in getattr(self, "__{}".format(hook))
                         if h.file_name == file_name]
        return all_hooks

    def get_all_methods_in(self, file_name):
        methods = []
        for _, infos in self.__steps_map.items():
            methods = methods + [i for i in infos if i.file_name == file_name]
        return methods + self._get_all_hooks(file_name)

    def is_file_cached(self, file_name):
        for _, infos in self.__steps_map.items():
            if any(i.file_name == file_name for i in infos):
                return True
        return False

    def remove_steps(self, file_name):
        new_map = {}
        for step, infos in self.__steps_map.items():
            filtered_info = [i for i in infos if i.file_name != file_name]
            if len(filtered_info) > 0:
                new_map[step] = filtered_info
        self.__steps_map = new_map

    def clear(self):
        self.__steps_map, self.__continue_on_failures = {}, {}
        for hook in Registry.hooks:
            setattr(self, '__{}'.format(hook), [])


def _filter_hooks(tags, hooks):
    filtered_hooks = []
    for hook in hooks:
        hook_tags = hook.tags
        if hook_tags is None:
            filtered_hooks.append(hook)
            continue
        for tag in tags:
            hook_tags = hook_tags.replace('<{}>'.format(tag), 'True')
        if eval(re.sub('<[^<]+?>', 'False', hook_tags)):
            filtered_hooks.append(hook)
    return filtered_hooks


def _get_step_value(step_text):
    return re.sub('(<.*?>)', '{}', step_text)


def _take_screenshot():
    temp_file = os.path.join(tempfile.gettempdir(), 'screenshot.png')
    try:
        call(['gauge_screenshot', temp_file])
        _file = open(temp_file, 'r+b')
        data = _file.read()
        _file.close()
        return data
    except Exception as err:
        logger.error(
            "\nFailed to take screenshot using gauge_screenshot.\n{0}".format(err))
    except:
        logger.error("\nFailed to take screenshot using gauge_screenshot.\n{0}".format(
            sys.exc_info()[0]))
    return str.encode("")


registry = Registry()


class ScreenshotsStore:
    __screenshots = []

    @staticmethod
    def pending_screenshots():
        screenshots = ScreenshotsStore.__screenshots
        ScreenshotsStore.__screenshots = []
        return screenshots

    @staticmethod
    def capture():
        ScreenshotsStore.__screenshots.append(registry.screenshot_provider()())

    @staticmethod
    def clear():
        ScreenshotsStore.__screenshots = []



import json
import sys


def debug(message):
    _print('debug', message)


def info(message):
    _print('info', message)


def error(message):
    _print('error', message, True)


def warning(message):
    _print('warning', message)


def fatal(message):
    _print('fatal', message, True)


def _print(level, message, is_error=False):
    stream = sys.stdout if not is_error else sys.stderr
    stream.write("{}\n".format(json.dumps({"logLevel": level, "message": message})))



import importlib
import inspect
import json
import re
import shutil
import sys
import traceback
from os import path

from getgauge import logger
from getgauge.registry import registry
from getgauge.util import *

project_root = get_project_root()
impl_dirs = get_step_impl_dirs()
env_dir = os.path.join(project_root, 'env', 'default')
requirements_file = os.path.join(project_root, 'requirements.txt')
sys.path.append(project_root)
PLUGIN_JSON = 'python.json'
VERSION = 'version'
PYTHON_PROPERTIES = 'python.properties'
SKEL = 'skel'


def load_impls(step_impl_dirs=impl_dirs):
    os.chdir(project_root)
    for impl_dir in step_impl_dirs:
        if not os.path.isdir(impl_dir):
            logger.error('Cannot import step implementations. Error: {} does not exist.'.format(step_impl_dirs))
            logger.error('Make sure `STEP_IMPL_DIR` env var is set to a valid directory path.')
            return
        base_dir = project_root if impl_dir.startswith(project_root) else os.path.dirname(impl_dir)
        _import_impl(base_dir, impl_dir)


def copy_skel_files():
    try:
        logger.info('Initialising Gauge Python project')
        logger.info('create  {}'.format(env_dir))
        os.makedirs(env_dir)
        logger.info('create  {}'.format(impl_dirs[0]))
        shutil.copytree(os.path.join(SKEL,path.basename(impl_dirs[0]) ), impl_dirs[0])
        logger.info('create  {}'.format(os.path.join(env_dir, PYTHON_PROPERTIES)))
        shutil.copy(os.path.join(SKEL, PYTHON_PROPERTIES), env_dir)
        f = open(requirements_file, 'w')
        f.write('getgauge==' + _get_version())
        f.close()
    except:
        logger.fatal('Exception occurred while copying skel files.\n{}.'.format(traceback.format_exc()))


def _import_impl(base_dir, step_impl_dir):
    for f in os.listdir(step_impl_dir):
        file_path = os.path.join(step_impl_dir, f)
        if f.endswith('.py'):
            _import_file(base_dir, file_path)
        elif path.isdir(file_path):
            _import_impl(base_dir, file_path)


def _import_file(base_dir, file_path):
    rel_path = os.path.normpath(file_path.replace(base_dir + os.path.sep, ''))
    try:
        module_name = os.path.splitext(rel_path.replace(os.path.sep, '.'))[0]
        m = importlib.import_module(module_name)
        # Get all classes in the imported module
        classes = inspect.getmembers(m, lambda member: inspect.isclass(member) and member.__module__ == module_name)
        if len(classes) > 0:
            for c in classes:
                file = inspect.getfile(c[1])
                # Create instance of step implementation class.
                if _has_methods_with_gauge_decoratores(c[1]):
                    update_step_resgistry_with_class(c[1](), file_path) # c[1]() will create a new instance of the class
    except:
        logger.error('Exception occurred while loading step implementations from file: {}.'.format(rel_path))
        logger.error(traceback.format_exc())

# Inject instace in each class method (hook/step)
def update_step_resgistry_with_class(instance, file_path):
    for info in registry.get_all_methods_in(file_path):
        class_methods = [x[0] for x in inspect.getmembers(instance, inspect.ismethod)]
        if info.impl.__name__ in class_methods:
            info.instance = instance


def _get_version():
    json_data = open(PLUGIN_JSON).read()
    data = json.loads(json_data)
    return data[VERSION]

def _has_methods_with_gauge_decoratores(klass):
    foo = r"@(step|before_suite|after_suite|before_scenario|after_scenario|before_spec|after_spec|before_step|after_step|screenshot|custom_screen_grabber)"
    sourcelines = inspect.getsourcelines(klass)[0]
    for i,line in enumerate(sourcelines):
        if re.match(foo, line.strip()) != None:
            return True



from getgauge.registry import registry
from getgauge.parser import PythonFile
from getgauge.messages.spec_pb2 import Span
from getgauge.messages.messages_pb2 import TextDiff


def refactor_step(request, response, with_location=True):
    if registry.has_multiple_impls(request.oldStepValue.stepValue):
        raise Exception('Multiple Implementation found for `{}`'.format(
            request.oldStepValue.parameterizedStepValue
        ))
    info = registry.get_info_for(request.oldStepValue.stepValue)
    impl_file = PythonFile.parse(info.file_name)
    diffs = impl_file.refactor_step(
        info.step_text,
        request.newStepValue.parameterizedStepValue,
        _new_parameter_positions(request),
    )
    content = impl_file.get_code()
    if request.saveChanges:
        with open(info.file_name, 'w') as f:
            f.write(content)
    response.refactorResponse.success = True
    response.refactorResponse.filesChanged.append(info.file_name)
    response.refactorResponse.fileChanges.add(
        fileName=info.file_name,
        fileContent=content,  # FIXME: Remove deprecated field
        diffs=[TextDiff(span=Span(**d[0]), content=d[1]) for d in diffs],
    )


def _new_parameter_positions(request):
    moved_pos = list(range(len(request.paramPositions)))
    for index, position in enumerate(request.paramPositions):
        moved_pos[position.newPosition] = position.oldPosition
    return moved_pos



import ast

import parso
import six
from getgauge import logger

# Reuse parser for multiple invocations. This also prevents
# problems with pyfakefs during testing for Python 3.7
_parser = parso.load_grammar()


class ParsoPythonFile(object):

    @staticmethod
    def parse(file_path, content=None):
        """
        Create a PythonFile object with specified file_path and content.
        If content is None then, it is loaded from the file_path method.
        Otherwise, file_path is only used for reporting errors.
        """
        try:
            # Parso reads files in binary mode and converts to unicode
            # using python_bytes_to_unicode() function. As a result,
            # we no longer have information about original file encoding and
            # output of module.get_content() can't be converted back to bytes
            # For now we can make a compromise by reading the file ourselves
            # and passing content to parse() function.
            if content is None:
                with open(file_path) as f:
                    content = f.read()
            py_tree = _parser.parse(
                content, path=file_path, error_recovery=False)
            return ParsoPythonFile(file_path, py_tree)
        except parso.parser.ParserSyntaxError as ex:
            logger.error("Failed to parse {0}:{1} '{2}'".format(file_path, ex.error_leaf.line, ex.error_leaf.get_code()))

    def __init__(self, file_path, py_tree):
        self.file_path = file_path
        self.py_tree = py_tree

    def _span_from_pos(self, start_pos, end_pos):
        return {
            'start': start_pos[0],
            'startChar': start_pos[1],
            'end': end_pos[0],
            'endChar': end_pos[1],
        }

    def _iter_step_func_decorators(self):
        """Find functions with step decorator in parsed file"""
        func_defs = [func for func in self.py_tree.iter_funcdefs()] + [func for cls in self.py_tree.iter_classdefs() for func in cls.iter_funcdefs()]
        for func in func_defs:
            for decorator in func.get_decorators():
                try:
                    if decorator.children[1].value == 'step':
                        yield func, decorator
                        break
                except AttributeError:
                        continue

    def _step_decorator_args(self, decorator):
        """
        Get the arguments passed to step decorators
        converted to python objects.
        """
        args = decorator.children[3:-2]
        step = None
        if len(args) == 1:
            try:
                step = ast.literal_eval(args[0].get_code())
            except (ValueError, SyntaxError):
                pass
            if isinstance(step, six.string_types+(list,)):
                return step
            logger.error("Decorator step accepts either a string or a list of strings - {1}:{0}".format(self.file_path, decorator.start_pos[0]))
        else:
            logger.error("Decorator step accepts only one argument - {0}:{1}".format(self.file_path, decorator.start_pos[0]))

    def iter_steps(self):
        """Iterate over steps in the parsed file."""
        for func, decorator in self._iter_step_func_decorators():
            step = self._step_decorator_args(decorator)
            if step:
                span = self._span_from_pos(decorator.start_pos, func.end_pos)
                yield step, func.name.value, span

    def _find_step_node(self, step_text):
        """Find the ast node which contains the text."""
        for func, decorator in self._iter_step_func_decorators():
            step = self._step_decorator_args(decorator)
            arg_node = decorator.children[3]
            if step == step_text:
                return arg_node, func
            elif isinstance(step, list) and step_text in step:
                idx = step.index(step_text)
                step_node = arg_node.children[1].children[idx * 2]
                return step_node, func
        return None, None

    def _refactor_step_text(self, step, old_text, new_text):
        step_span = self._span_from_pos(step.start_pos, step.end_pos)
        step.value = step.value.replace(old_text, new_text)
        return step_span, step.value

    def _create_param_node(self, parent, name, prefix, is_last):
        start_pos = parent[-1].end_pos[0], parent[-1].end_pos[1] + len(prefix)
        children = [parso.python.tree.Name(name, start_pos, prefix)]
        if not is_last:
            children.append(parso.python.tree.Operator(
                ',', children[-1].end_pos))
        return parso.python.tree.Param(children, parent)

    def _move_param_nodes(self, param_nodes, move_param_from_idx):
        # Param nodes include opening and closing braces
        num_params = len(param_nodes) - 2
        # If the move list is exactly same as current params
        # list then no need to create a new list.
        if list(range(num_params)) == move_param_from_idx:
            return param_nodes
        # Get the prefix from second parameter to use with new parameters
        prefix = param_nodes[2].name.prefix if num_params > 1 else ' '
        new_param_nodes = [parso.python.tree.Operator(
            '(', param_nodes[0].start_pos)]
        for i, move_from in enumerate(move_param_from_idx):
            param = self._create_param_node(
                new_param_nodes,
                self._get_param_name(
                    param_nodes, i) if move_from < 0 else param_nodes[move_from + 1].name.value,
                '' if i == 0 else prefix,
                i >= len(move_param_from_idx) - 1
            )
            new_param_nodes.append(param)
        new_param_nodes.append(parso.python.tree.Operator(
            ')', new_param_nodes[-1].end_pos))
        # Change the parent to actual function
        for node in new_param_nodes:
            node.parent = param_nodes[0].parent
        return new_param_nodes

    def refactor_step(self, old_text, new_text, move_param_from_idx):
        """
        Find the step with old_text and change it to new_text.The step function
        parameters are also changed according to move_param_from_idx.
        Each entry in this list should specify parameter position from old.
        """
        diffs = []
        step, func = self._find_step_node(old_text)
        if step is None:
            return diffs
        step_diff = self._refactor_step_text(step, old_text, new_text)
        diffs.append(step_diff)
        params_list_node = func.children[2]
        moved_params = self._move_param_nodes(
            params_list_node.children, move_param_from_idx)
        if params_list_node.children is not moved_params:
            # Record original parameter list span excluding braces
            params_span = self._span_from_pos(
                params_list_node.children[0].end_pos,
                params_list_node.children[-1].start_pos)
            params_list_node.children = moved_params
            # Get code for moved paramters excluding braces
            param_code = ''.join(p.get_code() for p in moved_params[1:-1])
            diffs.append((params_span, param_code))
        return diffs

    def get_code(self):
        """Return current content of the tree."""
        return self.py_tree.get_code()

    def _get_param_name(self, param_nodes, i):
        name = 'arg{}'.format(i)
        if name not in [x.name.value for x in param_nodes[1:-1]]:
            return name
        else:
            return self._get_param_name(param_nodes, i + 1)



import os
import six
from abc import ABCMeta, abstractmethod
from getgauge.parser_parso import ParsoPythonFile
from getgauge.parser_redbaron import RedbaronPythonFile


class PythonFile(object):
    Class = None

    @staticmethod
    def parse(file_path, content=None):
        """
        Create a PythonFileABC object with specified file_path and content. If content is None
        then, it is loaded from the file_path method. Otherwise, file_path is only used for
        reporting errors.
        """
        return PythonFile.Class.parse(file_path, content)

    @staticmethod
    def select_python_parser(parser=None):
        """
        Select default parser for loading and refactoring steps. Passing `redbaron` as argument
        will select the old paring engine from v0.3.3

        Replacing the redbaron parser was necessary to support Python 3 syntax. We have tried our
        best to make sure there is no user impact on users. However, there may be regressions with
        new parser backend.

        To revert to the old parser implementation, add `GETGAUGE_USE_0_3_3_PARSER=true` property
        to the `python.properties` file in the `<PROJECT_DIR>/env/default directory.

        This property along with the redbaron parser will be removed in future releases.
        """
        if parser == 'redbaron' or os.environ.get('GETGAUGE_USE_0_3_3_PARSER'):
            PythonFile.Class = RedbaronPythonFile
        else:
            PythonFile.Class = ParsoPythonFile


# Select the default implementation
PythonFile.select_python_parser()


class PythonFileABC(six.with_metaclass(ABCMeta)):
    @staticmethod
    def parse(file_path, content=None):
        """
        Create a PythonFileABC object with specified file_path and content. If content is None
        then, it is loaded from the file_path method. Otherwise, file_path is only used for
        reporting errors.
        """
        raise NotImplementedError

    @abstractmethod
    def iter_steps(self):
        """Iterate over steps in the parsed file"""
        raise NotImplementedError

    @abstractmethod
    def refactor_step(self, old_text, new_text, move_param_from_idx):
        """
        Find the step with old_text and change it to new_text. The step function
        parameters are also changed according to move_param_from_idx. Each entry in
        this list should specify parameter position from old
        """
        raise NotImplementedError

    @abstractmethod
    def get_code(self):
        """Returns current content of the tree."""
        raise NotImplementedError


# Verify that implemetations are subclasses of ABC
PythonFileABC.register(ParsoPythonFile)
PythonFileABC.register(RedbaronPythonFile)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lsp.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import getgauge.messages.messages_pb2 as messages__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='lsp.proto',
  package='gauge.messages',
  syntax='proto3',
  serialized_pb=_b('\n\tlsp.proto\x12\x0egauge.messages\x1a\x0emessages.proto\"\x07\n\x05\x45mpty2\xea\x06\n\nlspService\x12S\n\x0cGetStepNames\x12 .gauge.messages.StepNamesRequest\x1a!.gauge.messages.StepNamesResponse\x12\x44\n\tCacheFile\x12 .gauge.messages.CacheFileRequest\x1a\x15.gauge.messages.Empty\x12_\n\x10GetStepPositions\x12$.gauge.messages.StepPositionsRequest\x1a%.gauge.messages.StepPositionsResponse\x12_\n\x16GetImplementationFiles\x12\x15.gauge.messages.Empty\x1a..gauge.messages.ImplementationFileListResponse\x12X\n\rImplementStub\x12-.gauge.messages.StubImplementationCodeRequest\x1a\x18.gauge.messages.FileDiff\x12Y\n\x0cValidateStep\x12#.gauge.messages.StepValidateRequest\x1a$.gauge.messages.StepValidateResponse\x12M\n\x08Refactor\x12\x1f.gauge.messages.RefactorRequest\x1a .gauge.messages.RefactorResponse\x12P\n\x0bGetStepName\x12\x1f.gauge.messages.StepNameRequest\x1a .gauge.messages.StepNameResponse\x12_\n\x0fGetGlobPatterns\x12\x15.gauge.messages.Empty\x1a\x35.gauge.messages.ImplementationFileGlobPatternResponse\x12H\n\x0bKillProcess\x12\".gauge.messages.KillProcessRequest\x1a\x15.gauge.messages.EmptyB)\n\x16\x63om.thoughtworks.gauge\xaa\x02\x0eGauge.Messagesb\x06proto3')
  ,
  dependencies=[messages__pb2.DESCRIPTOR,])




_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='gauge.messages.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=45,
  serialized_end=52,
)

DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), dict(
  DESCRIPTOR = _EMPTY,
  __module__ = 'lsp_pb2'
  # @@protoc_insertion_point(class_scope:gauge.messages.Empty)
  ))
_sym_db.RegisterMessage(Empty)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\026com.thoughtworks.gauge\252\002\016Gauge.Messages'))

_LSPSERVICE = _descriptor.ServiceDescriptor(
  name='lspService',
  full_name='gauge.messages.lspService',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=55,
  serialized_end=929,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetStepNames',
    full_name='gauge.messages.lspService.GetStepNames',
    index=0,
    containing_service=None,
    input_type=messages__pb2._STEPNAMESREQUEST,
    output_type=messages__pb2._STEPNAMESRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CacheFile',
    full_name='gauge.messages.lspService.CacheFile',
    index=1,
    containing_service=None,
    input_type=messages__pb2._CACHEFILEREQUEST,
    output_type=_EMPTY,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetStepPositions',
    full_name='gauge.messages.lspService.GetStepPositions',
    index=2,
    containing_service=None,
    input_type=messages__pb2._STEPPOSITIONSREQUEST,
    output_type=messages__pb2._STEPPOSITIONSRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetImplementationFiles',
    full_name='gauge.messages.lspService.GetImplementationFiles',
    index=3,
    containing_service=None,
    input_type=_EMPTY,
    output_type=messages__pb2._IMPLEMENTATIONFILELISTRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ImplementStub',
    full_name='gauge.messages.lspService.ImplementStub',
    index=4,
    containing_service=None,
    input_type=messages__pb2._STUBIMPLEMENTATIONCODEREQUEST,
    output_type=messages__pb2._FILEDIFF,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ValidateStep',
    full_name='gauge.messages.lspService.ValidateStep',
    index=5,
    containing_service=None,
    input_type=messages__pb2._STEPVALIDATEREQUEST,
    output_type=messages__pb2._STEPVALIDATERESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Refactor',
    full_name='gauge.messages.lspService.Refactor',
    index=6,
    containing_service=None,
    input_type=messages__pb2._REFACTORREQUEST,
    output_type=messages__pb2._REFACTORRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetStepName',
    full_name='gauge.messages.lspService.GetStepName',
    index=7,
    containing_service=None,
    input_type=messages__pb2._STEPNAMEREQUEST,
    output_type=messages__pb2._STEPNAMERESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetGlobPatterns',
    full_name='gauge.messages.lspService.GetGlobPatterns',
    index=8,
    containing_service=None,
    input_type=_EMPTY,
    output_type=messages__pb2._IMPLEMENTATIONFILEGLOBPATTERNRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='KillProcess',
    full_name='gauge.messages.lspService.KillProcess',
    index=9,
    containing_service=None,
    input_type=messages__pb2._KILLPROCESSREQUEST,
    output_type=_EMPTY,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_LSPSERVICE)

DESCRIPTOR.services_by_name['lspService'] = _LSPSERVICE

# @@protoc_insertion_point(module_scope)



import os
from getgauge.registry import registry
from getgauge.parser import PythonFile


def load_steps(python_file):
    for funcStep in python_file.iter_steps():
        registry.add_step(funcStep[0], funcStep[1],
                          python_file.file_path, funcStep[2])


def reload_steps(file_path, content=None):
    pf = PythonFile.parse(file_path, content)
    if pf:
        registry.remove_steps(file_path)
        load_steps(pf)


def load_files(step_impl_dirs):
    for step_impl_dir in step_impl_dirs:
        for dirpath, _, files in os.walk(step_impl_dir):
                py_files = (os.path.join(dirpath, f) for f in files if f.endswith('.py'))
                for file_path in py_files:
                        pf = PythonFile.parse(file_path)
                        if pf:
                                load_steps(pf)
